@@ -1,3 +1,7 @@
+test_that("can access nickname", {
+  expect_snapshot(version$nickname, variant = r_version())
+})
+
 test_that("can snapshot output", {
   foo <- function() cat("y")
   expect_snapshot_output(foo())
@@ -25,6 +29,20 @@ test_that("multiple outputs of same type are collapsed", {
   })
 })
 
+test_that("can scrub output/messages/warnings/errors", {
+  secret <- function() {
+    print("secret")
+    message("secret")
+    warn("secret")
+    abort("secret")
+  }
+  redact <- function(x) gsub("secret", "<redacted>", x)
+  expect_snapshot(secret(), transform = redact, error = TRUE)
+
+  # Or with an inline fun
+  expect_snapshot(print("secret"), transform = ~ gsub("secret", "****", .x))
+})
+
 test_that("always checks error status", {
   expect_error(expect_snapshot(stop("!"), error = FALSE))
   expect_failure(expect_snapshot(print("!"), error = TRUE))
@@ -72,8 +90,13 @@ test_that("can control snapshot value details", {
   expect_snapshot_value(1.2, tolerance = 0.1)
 })
 
+test_that("tolerance passed to check_roundtrip", {
+  expect_snapshot_value(0.900000000000001, style = "json")
+})
+
 test_that("reparse handles common cases", {
   roundtrip <- function(x) reparse(deparse(x))
+  expect_equal(roundtrip(-1), -1)
   expect_equal(roundtrip(c(1, 2, 3)), c(1, 2, 3))
   expect_equal(roundtrip(list(1, 2, 3)), list(1, 2, 3))
   expect_equal(roundtrip(mtcars), mtcars)
